@@ -9,10 +9,11 @@ from typing import List
 
 from quid.core.Quid import Quid
 from quid.helper.Loader import load_matches
+from quid.match.MatchSpan import MatchSpan
 
-from proquo.core.MatchRef import MatchRef
+from proquo.match.MatchRef import MatchRef
 from proquo.core.ProQuo import ProQuo
-from proquo.model.reference.ReferenceModel import ReferenceModel
+from proquo.model.reference.ReferenceModelTrainer import ReferenceModelTrainer
 from proquo.model.relation.RelationVectorizerBert import RelationVectorizerBert
 import transformers
 
@@ -21,6 +22,8 @@ from proquo.testing.reference import TestReference
 from proquo.testing.relation import TestRelationBert
 from proquo.training.reference import TrainReference
 from proquo.training.relation import TrainRelationBert
+import re
+import json
 
 # only import if tensorflow_text is found. This is hardly ever needed and tensorflow_text is not always easy to install.
 try:
@@ -30,25 +33,89 @@ except ModuleNotFoundError:
     pass
 
 
+def __json_encoder_proquo(obj):
+    if isinstance(obj, MatchRef):
+        result_dict = obj.__dict__
+
+        if not result_dict['reference']:
+            del result_dict['reference']
+
+        return result_dict
+
+    if isinstance(obj, MatchSpan):
+        result_dict = obj.__dict__
+
+        if not result_dict['text']:
+            del result_dict['text']
+
+        return result_dict
+
+    return obj.__dict__
+
+
 def __process_file(pro_quo, filename, source_file_content, target_file_content, quid_matches, output_folder_path,
-                   source_text_parallel_print):
+                   source_text_parallel_print, export_text, export_ref, output_type, csv_sep):
 
     short_matches: List[MatchRef] = pro_quo.compare(source_file_content, target_file_content, quid_matches,
                                                     source_text_parallel_print)
 
-    # todo: an Quid anpassen
-    result = ''
+    if not export_text:
+        for match in short_matches:
+            match.source_span.text = ''
+            match.target_span.text = ''
 
-    for match in short_matches:
-        result += f'\n{match.source_span.start}\t{match.source_span.end}' \
-                  f'\t{match.target_span.start}\t{match.target_span.end}' \
-                  f'\t{match.source_span.text}\t{match.target_span.text}'
+    if not export_ref:
+        for match in short_matches:
+            match.reference = None
 
-        if match.reference:
-            result += f'\t{match.reference.start}\t{match.reference.end}\t{match.reference.text}'
+    if output_type == 'json':
+        result = json.dumps(short_matches, default=__json_encoder_proquo)
+        file_ending = 'json'
+    elif output_type == 'csv':
+        result = f'sstart{csv_sep}send{csv_sep}tstart{csv_sep}tend{csv_sep}stext{csv_sep}ttext'
+
+        if export_ref:
+            result += f'{csv_sep}rstart{csv_sep}rend{csv_sep}rtext'
+
+        for match in short_matches:
+            source_span = match.source_span
+            target_span = match.target_span
+
+            result += f'\n{source_span.start}{csv_sep}{source_span.end}' \
+                      f'{csv_sep}{target_span.start}{csv_sep}{target_span.end}'
+
+            if export_text:
+                source_span_text = re.sub(rf'[{csv_sep}\n]', ' ', source_span.text)
+                target_span_text = re.sub(rf'[{csv_sep}\n]', ' ', target_span.text)
+                result += f'{csv_sep}{source_span_text}{csv_sep}{target_span_text}'
+
+            if export_ref and match.reference:
+                ref_text = re.sub(rf'[{csv_sep}\n]', ' ', match.reference.text)
+                result += f'{csv_sep}{match.reference.start}{csv_sep}{match.reference.end}{csv_sep}{ref_text}'
+            elif export_ref:
+                result += f'{csv_sep}{csv_sep}{csv_sep}'
+
+        file_ending = 'csv'
+    else:
+        result = ''
+
+        for match in short_matches:
+            result += f'\n{match.source_span.start}\t{match.source_span.end}' \
+                      f'\t{match.target_span.start}\t{match.target_span.end}'
+
+            if export_text:
+                result += f'\t{match.source_span.text}\t{match.target_span.text}'
+
+            if export_ref and match.reference:
+                result += f'\t{match.reference.start}\t{match.reference.end}\t{match.reference.text}'
+
+        result = result.strip()
+        file_ending = 'txt'
 
     if output_folder_path:
-        with open(join(output_folder_path, filename + '.csv'), 'w', encoding='utf-8') as output_file:
+        filename = f'{filename}.{file_ending}'
+
+        with open(join(output_folder_path, filename), 'w', encoding='utf-8') as output_file:
             output_file.write(result)
     else:
         print('Results:')
@@ -79,12 +146,13 @@ def __test_relation_bert(test_file_path, tokenizer_folder_path, model_folder_pat
 
 
 def __run_compare(source_file_path, target_path, ref_vocab_file_path, ref_model_file_path, rel_tokenizer_folder_path,
-                  rel_model_folder_path, quid_match_path, output_folder_path, parallel_print_files):
+                  rel_model_folder_path, quid_match_path, output_folder_path, parallel_print_files, export_text,
+                  export_ref, output_type, csv_sep):
 
     reference_vectorizer = ReferenceVectorizer.from_vocab_file(ref_vocab_file_path, 25, True)
-    reference_model = ReferenceModel(25, True, 32, 32, 0.2, 512, 10)
-    reference_keras_model = reference_model.get_model(reference_vectorizer.max_id)
-    reference_keras_model.load_weights(ref_model_file_path)
+    reference_model_trainer = ReferenceModelTrainer(25, True, 32, 32, 0.2, 512, 10)
+    reference_model = reference_model_trainer.get_model(reference_vectorizer.max_id)
+    reference_model.load_weights(ref_model_file_path)
 
     relation_vectorizer = RelationVectorizerBert.from_saved(300, rel_tokenizer_folder_path, True)
     relation_model = transformers.TFBertForSequenceClassification.from_pretrained(rel_model_folder_path, num_labels=2)
@@ -92,7 +160,7 @@ def __run_compare(source_file_path, target_path, ref_vocab_file_path, ref_model_
     with open(source_file_path, 'r', encoding='utf-8') as source_file:
         source_file_content = source_file.read().lower()
 
-    pro_quo = ProQuo(reference_keras_model, reference_vectorizer, relation_model, relation_vectorizer)
+    pro_quo = ProQuo(reference_model, reference_vectorizer, relation_model, relation_vectorizer)
 
     if isfile(target_path) and target_path.endswith('.txt'):
         with open(target_path, 'r', encoding='utf-8') as target_file:
@@ -112,7 +180,7 @@ def __run_compare(source_file_path, target_path, ref_vocab_file_path, ref_model_
             source_text_parallel_print = True
 
         __process_file(pro_quo, filename, source_file_content, target_file_content, quid_matches, output_folder_path,
-                       source_text_parallel_print)
+                       source_text_parallel_print, export_text, export_ref, output_type, csv_sep)
     elif isdir(target_path):
         for fileOrFolder in listdir(target_path):
             target_file_path = join(target_path, fileOrFolder)
@@ -136,20 +204,28 @@ def __run_compare(source_file_path, target_path, ref_vocab_file_path, ref_model_
                     source_text_parallel_print = True
 
                 __process_file(pro_quo, filename, source_file_content, target_file_content, quid_matches,
-                               output_folder_path, source_text_parallel_print)
+                               output_folder_path, source_text_parallel_print, export_text, export_ref, output_type,
+                               csv_sep)
 
 
 def main(argv=None):
-    train_description = 'TBD'
-    train_reference_description = 'TBD'
-    train_relation_description = 'TBD'
-    test_description = 'TBD'
-    test_reference_description = 'TBD'
-    test_relation_description = 'TBD'
+    train_description = 'ProQuo train allows the user to train their own models.'
+    train_reference_description = 'ProQuo train reference allows the user to train their own reference model.'
+    train_relation_description = 'ProQuo train relation allows the user to train their own relation model.'
+    test_description = 'ProQuo test allows the user to test their trained models.'
+    test_reference_description = 'ProQuo test reference allows the user to test their trained reference model.'
+    test_relation_description = 'ProQuo test relation allows the user to test their trained relation model.'
+    test_relation_lstm_description = 'ProQuo test relation lstm allows the user to test their trained lstm relation' \
+                                     ' model.'
+    test_relation_bert_description = 'ProQuo test relation lstm allows the user to test their trained bert relation' \
+                                     ' model.'
 
-    compare_description = 'TBD'
+    compare_description = 'ProQuo compare allows the user to find short quotations (<= 4 words) in two texts, a' \
+                          ' source text and a target text. The target text is the text quoting the source text.' \
+                          ' Quotations in the target text need to be clearly marked with quotations marks.'
 
-    argument_parser = ArgumentParser(description='ProQuo is a tool to find (short) quotations in texts.')
+    argument_parser = ArgumentParser(prog='proquo', description='ProQuo is a tool to find short quotations (<= 4 words)'
+                                                                ' in texts.')
 
     subparsers_command = argument_parser.add_subparsers(dest='command')
     subparsers_command.required = True
@@ -189,7 +265,8 @@ def main(argv=None):
                                        help='Create a subfolder named with the current date to store the results')
     parser_train_relation.add_argument('--no-create-dated-subfolder', dest='create_dated_subfolder',
                                        action='store_false',
-                                       help='Do not create a subfolder named with the current date to store the results')
+                                       help='Do not create a subfolder named with the current date to store the '
+                                            'results')
     parser_train_relation.add_argument('--arch', choices=['lstm', 'bert'], dest='arch_type', default='bert',
                                        help='The model architecture to train')
 
@@ -213,8 +290,8 @@ def main(argv=None):
     subparsers_test_relation_arch = parser_test_relation.add_subparsers(dest='test_model_relation_arch')
     subparsers_test_relation_arch.required = True
 
-    parser_test_relation_lstm = subparsers_test_relation_arch.add_parser('lstm', help='',
-                                                                         description='')
+    parser_test_relation_lstm = subparsers_test_relation_arch.add_parser('lstm', help=test_relation_lstm_description,
+                                                                         description=test_relation_lstm_description)
 
     parser_test_relation_lstm.add_argument('test_file_path', nargs=1, metavar='test-file-path',
                                            help='Path to the txt file containing the testing examples')
@@ -223,8 +300,8 @@ def main(argv=None):
     parser_test_relation_lstm.add_argument('model_file_path', nargs=1, metavar='model-file-path',
                                            help='Path to the model file')
 
-    parser_test_relation_bert = subparsers_test_relation_arch.add_parser('bert', help='',
-                                                                         description='')
+    parser_test_relation_bert = subparsers_test_relation_arch.add_parser('bert', help=test_relation_bert_description,
+                                                                         description=test_relation_bert_description)
 
     parser_test_relation_bert.add_argument('test_file_path', nargs=1, metavar='test-file-path',
                                            help='Path to the txt file containing the testing examples')
@@ -260,6 +337,18 @@ def main(argv=None):
                                 action='store_false',
                                 help='Do not create a subfolder named with the current date to store the results')
     parser_compare.add_argument('--parallel-print-files', dest='parallel_print_files', nargs='*')
+    parser_compare.add_argument('--text', dest="export_text", default=True, action='store_true',
+                                help="Include matched text in the returned data structure")
+    parser_compare.add_argument('--no-text', dest='export_text', action='store_false',
+                                help="Don't include matched text in the returned data structure")
+    parser_compare.add_argument('--ref', dest="export_ref", default=False, action='store_true',
+                                help="Include matched reference in the returned data structure")
+    parser_compare.add_argument('--no-ref', dest='export_ref', action='store_false',
+                                help="Don't include matched reference in the returned data structure")
+    parser_compare.add_argument('--output-type', choices=['json', 'text', 'csv'], dest="output_type", default="json",
+                                help="The output type")
+    parser_compare.add_argument('--csv-sep', dest='csv_sep', type=str, help="output separator for csv (default: '\\t')",
+                                default='\t')
 
     args = argument_parser.parse_args(argv)
 
@@ -328,6 +417,10 @@ def main(argv=None):
         output_folder_path = args.output_folder_path
         create_dated_subfolder = args.create_dated_subfolder
         parallel_print_files = args.parallel_print_files
+        export_text = args.export_text
+        export_ref = args.export.ref
+        output_type = args.output_type
+        csv_sep = bytes(args.csv_sep, "utf-8").decode("unicode_escape")
 
         if create_dated_subfolder:
             now = datetime.now()
@@ -337,7 +430,7 @@ def main(argv=None):
 
         __run_compare(source_file_path, target_path, ref_vocab_file_path, ref_model_file_path,
                       rel_tokenizer_folder_path, rel_model_folder_path, quid_match_path, output_folder_path,
-                      parallel_print_files)
+                      parallel_print_files, export_text, export_ref, output_type, csv_sep)
 
 
 if __name__ == '__main__':
