@@ -5,7 +5,7 @@ from typing import Optional, List, Tuple
 from quid.match.Match import Match
 from quid.match.MatchSpan import MatchSpan
 
-from proquo.core.MatchRef import MatchRef
+from proquo.match.MatchRef import MatchRef
 from proquo.core.Quote import Quote
 from proquo.core.QuoteRef import QuoteRef
 from proquo.core.Reference import Reference
@@ -156,17 +156,20 @@ class ProQuo:
                                                                             len(source_text), page_size)
 
             quote_source_start = -1
+            quote_source_end = -1
             match_len = len(sq.quote.text.split())
 
             if match_len == 1:
                 if page_known_pos > -1:
-                    quote_source_start = self.__search_single_word(sq.quote, source_text, start, end, page_known_pos)
+                    quote_source_start, quote_source_end = self.__search_single_word(sq.quote, source_text, start, end,
+                                                                                     page_known_pos)
             else:
-                quote_source_start = self.__search_multi_word(short_matches, sq, start, end, source_text)
+                quote_source_start, quote_source_end = self.__search_multi_word(short_matches, sq, start, end,
+                                                                                source_text)
 
             if quote_source_start > -1:
-                quote_source_end = quote_source_start + len(sq.quote.text)
-                quote_source_text = ''
+                # quote_source_end = quote_source_start + len(sq.quote.text)
+                quote_source_text = source_text[quote_source_start:quote_source_end]
 
                 source_span = MatchSpan(quote_source_start, quote_source_end, quote_source_text)
                 target_span = MatchSpan(sq.quote.start, sq.quote.end, sq.quote.text)
@@ -654,7 +657,6 @@ class ProQuo:
         # word = self.__normalize_special_chars(word)
         # word = re.sub(' +', ' ', word, flags=re.DOTALL)
         # word = word.strip()
-
         # search_space = self.__normalize_special_chars(search_space)
 
         re_matches_iter = re.finditer(r'\b' + re.escape(word) + r'\b', search_space, flags=re.IGNORECASE)
@@ -801,10 +803,11 @@ class ProQuo:
             fuzzy_candidates = self.__fuzzy_match(quote.text, search_space_start, search_space_end)
 
             if len(fuzzy_candidates) == 1:
-                return fuzzy_candidates[0][0]
+                return fuzzy_candidates[0][0], fuzzy_candidates[0][1]
             elif len(fuzzy_candidates) > 1:
                 smallest_dist = sys.maxsize
                 best_match_start_pos = 0
+                best_match_end_pos = 0
 
                 for fc in fuzzy_candidates:
                     # TODO: Is start ideal? Maybe compare start and end
@@ -813,18 +816,20 @@ class ProQuo:
                     if diff < smallest_dist:
                         smallest_dist = diff
                         best_match_start_pos = m_start
+                        best_match_end_pos = fc[1]
 
-                return best_match_start_pos
+                return best_match_start_pos, best_match_end_pos
 
         elif strict_matches_count == 1:
             re_match = re_matches[0]
 
             if re_match:
-                return re_match.start() + search_space_start
+                return re_match.start() + search_space_start, re_match.end() + search_space_start
 
         elif page_known_pos > -1:
             smallest_dist = sys.maxsize
             best_match_start_pos = -1
+            best_match_end_pos = -1
 
             for m in re_matches:
                 # TODO: Is start ideal? Maybe compare start and end
@@ -833,42 +838,28 @@ class ProQuo:
                 if diff < smallest_dist:
                     smallest_dist = diff
                     best_match_start_pos = m_start
+                    best_match_end_pos = m.end() + search_space_start
 
-            return best_match_start_pos
+            return best_match_start_pos, best_match_end_pos
 
-        # TODO: put back in, for without ref only (?)
-        # search whole text as last try
-        # if mid_pos > -1:
-        #     re_matches = self.__strict_match(search_text, source_text)
-        #     strict_matches_count = len(re_matches)
-        #
-        #     if strict_matches_count == 1:
-        #         if re_matches[0]:
-        #             return re_matches[0].start()
-        #     elif strict_matches_count == 0:
-        #         fuzzy_candidates = self.__fuzzy_match(search_text, 0, len(source_text))
-        #
-        #         if len(fuzzy_candidates) == 1:
-        #             return fuzzy_candidates[0][0]
-
-        return -1
+        return -1, -1
 
     def __search_multi_word(self, short_matches, sq, range_start, range_end, source_text):
         search_space = source_text[range_start:range_end]
         re_matches = self.__strict_match(sq.quote.text, search_space)
 
         if len(re_matches) > 1:
-            return -1
+            return -1, -1
 
         if len(re_matches) == 1:
-            return re_matches[0].start() + range_start
+            return re_matches[0].start() + range_start, re_matches[0].end() + range_start
         elif len(re_matches) == 0:
             re_matches = self.__strict_match(sq.quote.text, source_text)
             if len(re_matches) > 1:
-                return -1
+                return -1, -1
 
             if len(re_matches) == 1:
-                return re_matches[0].start()
+                return re_matches[0].start(), re_matches[0].end()
 
         candidates = []
 
@@ -885,9 +876,9 @@ class ProQuo:
         candidates_count = len(candidates)
 
         if candidates_count == 0:
-            return -1
+            return -1, -1
         elif candidates_count == 1:
-            return candidates[0].source_span.start
+            return candidates[0].source_span.start, candidates[0].source_span.end
         else:
             filtered_candidates = []
 
@@ -896,6 +887,6 @@ class ProQuo:
                     filtered_candidates.append(c)
 
             if len(filtered_candidates) == 1:
-                return filtered_candidates[0].source_span.start
+                return filtered_candidates[0].source_span.start, filtered_candidates[0].source_span.end
 
-        return -1
+        return -1, -1
