@@ -1,6 +1,6 @@
 from os.path import join
 from pathlib import Path
-from proquo.model.linking.LinkingModel import LinkingModel
+from proquo.model.linking.LinkingModelTrainer import LinkingModelTrainer
 from datetime import datetime
 from proquo.model.linking.LinkingVectorizer import LinkingVectorizer
 
@@ -52,5 +52,5 @@ def train(train_file_path, val_file_path, output_path):
     Path(tokenizer_dir).mkdir(parents=True, exist_ok=True)
     linking_vectorizer.tokenizer.save_pretrained(tokenizer_dir)
 
-    model = LinkingModel(linking_vectorizer, 4, 3)
+    model = LinkingModelTrainer(linking_vectorizer, 4, 3)
     model.train_model(x_train, y_train, x_val, y_val, output_date_dir)
