@@ -1,6 +1,6 @@
 from os.path import join
 from pathlib import Path
-from proquo.model.relation.RelationModelBert import RelationModelBert
+from proquo.model.relation.RelationModelBertTrainer import RelationModelBertTrainer
 from proquo.model.relation.RelationVectorizerBert import RelationVectorizerBert
 
 
@@ -46,5 +46,5 @@ def train(train_file_path, val_file_path, output_folder_path):
     Path(tokenizer_dir).mkdir(parents=True, exist_ok=True)
     relation_vectorizer.tokenizer.save_pretrained(tokenizer_dir)
 
-    model = RelationModelBert(relation_vectorizer, 12, 3)
+    model = RelationModelBertTrainer(relation_vectorizer, 12, 3)
     model.train_model(x_train, y_train, x_val, y_val, output_folder_path)
