@@ -1,4 +1,4 @@
-from proquo.model.relation.RelationModelLstm import RelationModelLstm
+from proquo.model.relation.RelationModelLstmTrainer import RelationModelLstmTrainer
 
 
 def train(train_file_path, val_file_path, output_folder_path):
@@ -37,5 +37,5 @@ def train(train_file_path, val_file_path, output_folder_path):
             else:
                 print(f'wrong count: {line}')
 
-    model = RelationModelLstm(300, True, 64, 64, 0.3, 128, 5)
+    model = RelationModelLstmTrainer(300, True, 64, 64, 0.3, 128, 5)
     model.train_model(x_train, y_train, x_val, y_val, output_folder_path)
